import ij.ImageListener;
import ij.ImagePlus;
import ij.gui.ImageCanvas;
import ij.gui.Roi;

import java.awt.Canvas;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Iterator;

/**
* Synchronize ROIs in list of ImagePlus.
* @author Koutaro Abe ( kabe@cris.hokudai.ac.jp )
* @version 26-Oct-2010
*/
public class RoiSynchronizer implements ImageListener, MouseListener, KeyListener, MouseMotionListener {
	static ArrayList<Object> listeners;  // store instance of ImagePlus or of PixelInspector
	
	RoiSynchronizer( ArrayList<ImagePlus> list ) {
		listeners = new ArrayList<Object>();
		setListener( list );
	}
	void addListener( Object obj ){
//		IJ.log( "RoiSync.addlisteners:"+obj );
		if( obj==null )
			return;
		if( obj instanceof ImagePlus ) {
			ImagePlus imp = (ImagePlus)obj;
			Canvas ic = (Canvas) imp.getWindow().getCanvas();
			if( ic==null )
				return;
			listeners.add( imp );
//			IJ.log( "  added-->"+imp );
			MouseMotionListener[] m = ic.getMouseMotionListeners();
			addMMKListeners( ic );
//			IJ.log( "SyncRoi.addListener: get "+m.length+" MouseMotionListener." );
			for( int j=0; j<m.length; j++ )
				if( m[j] instanceof PixelInspector_ ) {
					PixelInspector_ p = (PixelInspector_)m[j];
					listeners.add( p );
//					IJ.log( "      added-->"+p );
					addMMKListeners( p.getCanvas() );
				}
		}
	}
	void setListener( ArrayList<ImagePlus> list ) {
		removeAllListeners();
		Iterator<ImagePlus> i = list.iterator();
		while( i.hasNext() ){
			addListener( i.next() );
//			IJ.log(" add "+imp.getTitle()+" Listener to SyncRoi." );
		}
	}
	void removeAllListeners() {
		if( listeners==null || listeners.size()==0 )
			return;
		Iterator<Object> i = listeners.iterator();
		while( i.hasNext() ){
			try {
				Object obj = i.next();
				if( obj instanceof ImagePlus )
					removeMMKListeners( ( (ImagePlus)obj ).getWindow().getCanvas() );
				else if( obj instanceof PixelInspector_ )
					removeMMKListeners( ( (PixelInspector_)obj ).getCanvas() );
			} catch( java.lang.NullPointerException e ) {
				continue;
			}
		}
		listeners.clear();
//		IJ.log( "RoiSync.removeAllListeners: removed." );
	}
	void addMMKListeners( Canvas ic ) {
		if( ic==null )
			return;
		ic.addMouseListener( this );
		ic.addMouseMotionListener( this );
		ic.addKeyListener( this );
	}
	void removeMMKListeners( Canvas ic ) {
		if( ic==null )
			return;
		ic.removeMouseListener( this );
		ic.removeMouseMotionListener( this );
		ic.removeKeyListener( this );
	}
	protected void notifyListeners( Object o ) {
		Roi roi;
//		IJ.log( "RoiSync.notifyListeners: source="+o );
		if( o instanceof ij.gui.ImageCanvas ) {
			o = ( (ImageCanvas)o ).getImage();
			roi = ((ImagePlus) o).getRoi();
		} else if( o instanceof PixelInspector_.InspectorCanvas ) {
			o = ((PixelInspector_.InspectorCanvas) o).getImage();
			roi = ((PixelInspector_) o).getRoi();
		} else
			return;
//		IJ.log( "  roi:source="+o );		
		synchronized( listeners ) {
			for ( int i=0; i<listeners.size(); i++ ) {
				Object l = listeners.get(i);
				if( o.equals( l ) ) {
//					IJ.log( "  skipped--"+o );
					continue;
				}
				if( l instanceof ImagePlus ) {
					( (ImagePlus)l ).setRoi( roi );
//					IJ.log( "  sent imp->"+l );
				}
				else if( l instanceof PixelInspector_ ) {
					( (PixelInspector_)l ).setRoi( roi );
//					IJ.log( "  sent pxi->"+l );
				}
			}
		}
	}
	public void imageClosed(ImagePlus imp) {}
	public void imageOpened(ImagePlus imp) {}
	public void imageUpdated(ImagePlus imp) { notifyListeners( imp ); }
	public void mouseClicked(MouseEvent e) { }
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { notifyListeners( e.getSource() ); }
	public void mouseDragged(MouseEvent e) { notifyListeners( e.getSource() ); }
	public void mouseMoved(MouseEvent e) {}
	public void keyPressed(KeyEvent e) { notifyListeners( e.getSource() ); }
	public void keyReleased(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {}
}
